	pro tom_demo,wave,obsspec,recspec,pshift,ratio,simspec

        ; Tomography parameter set up for spectral reconstruction. 
	; This procedure shows an example of how we dealt with 
	; red spectra of RY Persei (see Barai et al. 2004, ApJ, 608, 989).

        ; Output:
	;       wave    = waevlength grid for spectra 
        ;       obsspec = observed composite spectra
        ;       recspec = reconstructed spectra (current estimate) 
        ;       pshift  = orbital Doppler shifts for each spectrum, component
        ;       ratio   = flux ratio for each spectrum, component

	; get input spectra
	; Start with continuum rectified spectra of a binary. 
	; All the spectra should be placed on a common wavelength grid;
	; ideally the grid is spaced in log lambda so that each 
	; pixel spacing corresponds to the same Doppler velocity step. 
	; The procedure readstk.pro reads in two files:
	; ry_per.dat  = table of wavelengths and times of observation
	; ry_per.fits = grid of observed spectra 
	star='ry_per'
	readstk,wave,obsspec,hjd,star

	; select non-eclipse phase spectra
	; RY Per is an eclipsing binary so that at some orbital phases 
	; the flux is more dominated by one of the components. 
	; The flux contributions will differ from the simple assumptions
	; in these cases, so the simplest solution is to omit those 
	; particular observations.  Variable g below lists the indices
	; of the good (retained) spectra in the time index.  
	g=[13,14,15,16,19,20,21,22,23,24,25,26,27,28,29,30,31,34,35,36]
	obsspec=obsspec(*,g)
	hjd=hjd(g)

	; number of reconstructed components
	; The algorithm can be run for any number of components. 
	; Here we just consider two, the primary and secondary star spectra. 
	n=2
	npix=n_elements(wave)
	recspec=fltarr(npix,n)+1. ; set up matrix for reconstructions
	nsp=n_elements(hjd)
	pshift=fltarr(n,nsp)      ; pixel shifts for each component, spectrum
	ratio=fltarr(npix,n,nsp)+1.0  ; relative flux contributions for given
	; wavelength, component, and spectrum indices. 

	; set pshift from calculated orbits
	; ILST needs the orbital velocities from the outset. 
	; Here we read in the calculated radial velocities for the secondary
	; in each spectrum from output from the Chris Morbey program sbcm. 
	print,' Secondary component:' 
	readsbcm,'sbcm.out.sec',d2,c
	d2=d2(*,g)
	dvel=21.120012  ; pixel step in km/s for these spectra 
	for i=0,nsp-1 do pshift(1,i)=d2(2,i)/dvel
	; For RY Per, the secondary's radial velocity curve is better 
	; established than the primary's one, so the corresponding 
	; velocities of the primary are calculated from the orbital elements:
	v0=-5.9696  ; systemic velocity (km/s)
	k1=47.3509  ; primary semi-amplitude (km/s)
	k2=174.4652 ; secondary semi-amplitude (km/s) 
	for i=0,nsp-1 do pshift(0,i)=(-1.0*k1/k2*(d2(2,i)-v0)+v0)/dvel

	; set ratio
	; Need to establish the flux contibutions from the components
	; (normalized so that they sum to unity).  In the case of 
	; two hot stars observed in the Rayleigh-Jeans part of the 
	; spectrum, one number will suffice for this ratio. 
	; However, for RY Per, the components are quite different in 
	; temperature, so their relative spectral contributions vary 
	; significantly across the observed spectrum.   The version 
	; used below, tomography_rlambda.pro, handles this more 
	; complicated case.  Here we estimated the ratio from Kurucz 
	; flux models and stellar radii estimates for each wavelength 
	; point in the spectrum (stored in ratio1999m.fits). 
	r=readfits('ratio1999m.fits')
	for i=0,npix-1 do ratio(i,0,*)= 1.0/(1.+r(i))
	for i=0,npix-1 do ratio(i,1,*)=r(i)/(1.+r(i))

	; read in ism spectrum and remove from data
	; In many cases, the spectra will also contain lines formed 
	; in the interstellar medium that do not participate in the 
	; orbital motion of the stars.  The best approach is to remove 
	; these by division prior to spectral resonstruction. 
	; The file ismout.fits is an extracted ISM spectrum. 
	ism=readfits('ismout.fits')
	for i=0,nsp-1 do obsspec(0,i)=obsspec(*,i)/ism

	; advice
	; The algorithm is ready to run.  The first and second parameters
	; are the number of iterations and gain for each iteration, respectively. 
	; Details are given in Bagnuolo et al. 1994, ApJ, 423, 446. 
	; Here we use tomography_rlambda.pro that accounts for a wavelength
	; dependency in the flux ratio. 
	; The output is contained in the variables:
	; recspec   = reconstructed spectra of components, and 
	; simspec   = matrix of predicted composite spectra for each 
	;             time of observation. 
	print,' Running the tomography algorithm ... '
	print,' TOMOGRAPHY_RLAMBDA,100,0.95,obsspec,recspec,pshift,ratio,simspec'
	TOMOGRAPHY_RLAMBDA,100,0.95,obsspec,recspec,pshift,ratio,simspec
	plot,recspec(*,0)  ; plot primary spectrum (above)
	oplot,recspec(*,1)-0.5 ; plot secondary spectrum (below)
	; Note that the algorithm may introduce low frequency oscillations
	; in the results that may need to be divided out later (depending 
	; upon the application. 

	return
	end

@readstk.pro
@readsbcm.pro
@tomography_rlambda.pro

