	PRO TOMOGRAPHY_RLAMBDA,niters,delta,obsspec,recspec,pshift,ratio,simspec
	; IDL version of tomography algorithm based on Wiemker's thesis
	; this version uses wavelength variable ratio
	; Input:
	;	niters  = # of iterations
	;	delta   = convergence acceleration (0 to 1)
	;	obsspec = observed composite spectra
	;	recspec = reconstructed spectra (current estimate) 
	;	pshift  = pixel shifts for each spectrum, component
	;	ratio   = flux ratio for each pixel, spectrum, component
	; Output:
	;	recspec = reconstructed spectra (revised estimate) 
	;	simspec = composite spectra for same shifts as observed 

	np=n_elements(recspec(*,0))  ; # pixels
	nc=n_elements(recspec(0,*))  ; # stellar components
	ns=n_elements(obsspec(0,*))  ; # composite spectra 

	; Weighting factors
	is1=intarr(nc,ns)    ; integer shifts 
	is2=intarr(nc,ns)
	w1=fltarr(np,nc,ns)     ; fractional shifts
	w2=fltarr(np,nc,ns)
	wt1=fltarr(np,nc,ns)    ; weights
	wt2=fltarr(np,nc,ns)
	for j=0,ns-1 do begin   ; for each spectrum
	   for k=0,nc-1 do begin   ; for each component 
	      is1(k,j)=fix(pshift(k,j))
	      if (pshift(k,j) lt 0.) then is2(k,j)=is1(k,j)-1 else $
	         is2(k,j)=is1(k,j)+1 
	      w2(*,k,j)=abs(pshift(k,j)-is1(k,j))
	      w1(*,k,j)=1.-w2(*,k,j)
	      endfor
	   wtsum=total(w1(0,*,j)^2+w2(0,*,j)^2)
	   for k=0,nc-1 do begin 
	      for i=0,np-1 do begin 
	         wt1(i,k,j)=delta/ratio(i,k,j)*w1(i,k,j)/wtsum
	         wt2(i,k,j)=delta/ratio(i,k,j)*w2(i,k,j)/wtsum
	         w1(i,k,j)=ratio(i,k,j)*w1(i,k,j)
	         w2(i,k,j)=ratio(i,k,j)*w2(i,k,j)
	         endfor
	      endfor
	   endfor
	
	; Main iteration ...

	npp=np+100
	rs=fltarr(npp,nc)+1.     ; pad both sides with unity 
	cs=fltarr(npp,nc)
	resid=fltarr(npp,ns)     ; pad both sides with zeroes
	cr=fltarr(npp,ns)     
	top=np+49
	sample=indgen(np)

	; get extrapolated versions of w1,w2,wt1,wt2
	xpgrid=findgen(np)+50.
	xppgrid=findgen(npp)
	w1e=fltarr(npp,nc,ns)
	w2e=fltarr(npp,nc,ns)
	wt1e=fltarr(npp,nc,ns)
	wt2e=fltarr(npp,nc,ns)
	for j=0,ns-1 do begin   ; for each spectrum
	   for k=0,nc-1 do begin   ; for each component 
	      w1e(0,k,j)=interpol(w1(*,k,j),xpgrid,xppgrid)
	      w2e(0,k,j)=interpol(w2(*,k,j),xpgrid,xppgrid)
	      wt1e(0,k,j)=interpol(wt1(*,k,j),xpgrid,xppgrid)
	      wt2e(0,k,j)=interpol(wt2(*,k,j),xpgrid,xppgrid)
	      endfor
	   endfor

	for iiters=0,niters-1 do begin 
	   
	   for k=0,nc-1 do rs(50,k)=recspec(*,k)

	   ; form matrix of simulated spectra
	   simspec=0.*obsspec
	   for j=0,ns-1 do begin     ; compute each spectrum by
	      for k=0,nc-1 do begin  ; adding contributions from each component 
	         cs(0,k)=shift(w1e(*,k,j)*rs(*,k),is1(k,j)) + $
	                 shift(w2e(*,k,j)*rs(*,k),is2(k,j))
	         endfor 
	      css=rebin(cs,npp,1)*nc
	      simspec(0,j)=css(50:top)
	      ; calculate residuals
	      resid(50,j)=obsspec(*,j)-simspec(*,j) 
	      endfor

	   ; calculate corrections matrix 
	   for k=0,nc-1 do begin   ; for each component
	      ; get corrections from each spectrum
	      for j=0,ns-1 do $ 
	         cr(0,j)=shift(wt1e(*,k,j)*resid(*,j),-is1(k,j)) + $
	                 shift(wt2e(*,k,j)*resid(*,j),-is2(k,j))
	      recspec(0,k)=recspec(*,k)+rebin(cr(50:top,*),np,1)
	      endfor

	   ; enforce fluxes > 0
	   recspec=recspec > 0.

	   ; check rectification 
	   for k=0,nc-1 do begin
	      autorect,recspec(*,k),rectspec,sample,4,10,1,2
	      recspec(0,k)=rectspec
	      endfor
	   
	   endfor

	; final matrix of simulated spectra
	simspec=0.*obsspec
	for k=0,nc-1 do rs(50,k)=recspec(*,k)
	for j=0,ns-1 do begin     ; compute each spectrum by
	   for k=0,nc-1 do  $     ; adding contributions from each component 
	      cs(0,k)=shift(w1e(*,k,j)*rs(*,k),is1(k,j)) + $
	              shift(w2e(*,k,j)*rs(*,k),is2(k,j))
	   css=rebin(cs,npp,1)*nc
	   simspec(0,j)=css(50:top)
	   endfor

	return
	end

@/nfs/morgan4/gies/Idlpro/Tomography/autorect.pro
